#include-once

; #INDEX# =======================================================================================================================
; Title .........: _4X4
; COM Channel ...: 2
; AutoIt Version : 3.3.14
; Language ......: English
; Description ...: Control Of 4X4 HDMI Matrix Switch
;					For more details see file: AdditionalFiles\4X4 HDMI Matrix Switch serial protocol.docx
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
#ce

; #CURRENT# =====================================================================================================================
; _4X4open			- Open serial Port to Switch and retrieves current settings
; _4X4set			- Sets Selected Output to selected Input
; _4X4Send			- Send command to 4X4 switch via serial port opened by _4X4open
; _4X4Get			- gets a response to a command from the serial port
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Opens Serial Port to Switch then used _4X4read to retrieves current settings
; Parameters:		$aOutputs	- BYREF Array to place current settings into
; 						0 Based, Output 1 is element 0, Input 1 is expressed as 0
;					$nEDID		- EDID mode
; Returns:			Nothing values passed back in BYREF array
; Globals			$g_Config	- Configuration file to load settings from
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _4X4open(ByRef $aOutputs, ByRef $nEDID)
	_CommSwitch(2)																					;Use COM Channel 2 for 4X4 Matrix Switch
	_PortOpen("4X4")																				;Open Serial Port for Camera Control
	_4X4read($aOutputs, $nEDID)
	$nEDID = IniRead($g_Config,'4X4','EDID',-1)														;Any EDID value in INI file to set
	IF $nEDID >= 1 AND $nEDID <= 11 Then															;is value in range
		$EDID = Hex($nEDID+1,1)																		;convert to single HEX character 0 - A
		$Cmd = '05 55 18 0z 77'																		;build the command and send it
		$Cmd = StringReplace($Cmd,'z',$EDID)
		_4X4Send($Cmd)																				;Covert to byte values and send
	EndIf
EndFunc		;==>_4X4open

; #FUNCTION# ====================================================================================================================
; Description:		Reads current settings from switch
; Parameters:		$aOutputs	- BYREF Array to place current settings into
;	 					0 Based Output A is element 0 and input 1 is expressed as 0
;					$nEDID		- EDID mode
; Returns:			Nothing values passed back in BYREF array
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _4X4read(ByRef $aOutputs, ByRef $nEDID)
	_CommSwitch(2)																					;Use COM Channel 2 for 4X4 Matrix Switch
	_4X4clear()																						;clear initail power up data etc.
	_4X4send('04 55 20 77')																			;Inquire status of 4X4 Switch
	Local $FeedBack  = _4X4get(30)																	;Maxium characters to recieve
																									;format of return feedback
																									;\r\n s1x s2x s3x s4x EEx
	For $i = 0 to 3																					;Look for setting of each Output
		$p = StringInStr($FeedBack,'s' & Hex($i+1,1),1)												;Hex used to format to single character string
		IF $p > 0 Then																				;look or the sO where O is the output number
			$Input = StringMid($FeedBack,$p + 2,1)													;look for the very next character which is the input
			IF $Input > 3 Then																		;is input retrieved a
				$gV_CamStatus = 'Incorrect 4X4 Matrix Switch Feedback: ' & $FeedBack				;check the value
				$g_bStatus = True
				ExitLoop
			EndIf
			$aOutputs[$i] = $Input																	;Record the Input selected for that Output
		Else
			$gV_CamStatus = 'Incorrect 4X4 Matrix Switch Feedback: ' & $FeedBack					;Didn't find the channel setting in the string
			$g_bStatus = True
			ExitLoop
		EndIf
	Next
	$p = StringInStr($FeedBack,'EE',1)																;Look for the EDID Marker
	$c = StringMid($FeedBack,$p + 2,1)																;Get the EDID Character
	$nEDID = StringInStr('01234567890A',$c,1)														;Validate the character and adjust to the list entry number
	IF $nEDID = 0 Then
		$gV_CamStatus = 'Incorrect 4X4 Matrix Switch Feedback: ' & $FeedBack						;Didn't find the EDID setting in the string
	EndIf
	GUICtrlSetData($hEDID,$nEDID)
	$g_bStatus = True
	_CommSwitch(1)
EndFunc		;==>_4X4open

; #FUNCTION# ====================================================================================================================
; Description:		Sets selected Output to indicated Input
; Parameters:		$iOutputs	- Output number 0 based
;					$iInput		- Input number 0 based
; Returns:			Noting but sets GUI status line if any error sending command is incountered
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _4X4set($iOutput,$iInput)
	_CommSwitch(2)																					;Use COM Channel 2 for 4X4 Matrix Switch
	_4X4clear()
	Local $aPara[4][4] = [['09','1D','1F','0D'],['17','12','59','08'],['5E','06','05','03'],['18','44','0F','51']]
	Local $Cmd = '05 55 19 xx 77'
	$Cmd = StringReplace($Cmd,'xx',$aPara[$iOutput][$iInput],1)
	_4X4Send($Cmd)																					;Covert to byte values and send
	Local $FeedBack  = _4X4get(3)																	;Maxium characters to recieve
																									;format of return feedback
																									;sXY where X is Output -1 and Y is Input -1

	$p = StringInStr($FeedBack,'s')																	;Verify output set as expected
	IF $p > 0 Then
		IF StringMid($FeedBack,$p + 1,1) <> $iOutput + 1 OR StringMid($FeedBack,$p + 2,1) <> $iInput + 1 Then
			$gV_CamStatus = 'Invalid 4X4 Matrix Switch Feedback: ' & $FeedBack
			$g_bStatus = True
		EndIf
	Else
		$gV_CamStatus = 'No 4X4 Matrix Switch Feedback: ' & $FeedBack
		$g_bStatus = True
	EndIf
	_CommSwitch(1)
EndFunc		;==>_4X4set

; #FUNCTION# ====================================================================================================================
; Description:		Clears any pending data from 4X4 switch.
; Parameters:		None
; Returns:			Nothing will display Error dialogs if errors occur
; Author(s)			Paul verboom (pboom)
;
; Switch sends data in response to front panel key presses so unexpected data can be there, plus initial power up sends strings
; Best clear any data before sending any commands we are looking for response to so we only get the data we expect.
; Will time out after 1 second if full buffer can not be claered
; Does not set COM channel assumded taht is don by calling function.
; ===============================================================================================================================
Func _4X4clear()
	Local $ASCII = ''																				;used to recieve ASCII text response
	Local $Mess 																					;Any Error Message
	Local $iDelay = 50																				;How long to wait for incoming buffer clear out in milliseconds
	Local $hTimer = TimerInit()																		;Start a timer

	While True																						;loop to read in ACK messages from camera
		$NextChar = _CommReadByte(0)																;Get a byte a string conversion of number (eg '0', '20') with no waiting
		IF @error Then																				;@error = 1 No character recieved
			IF @error = 2 Then
				$Mess = 'DLL Call failed'
				ExitLoop
			EndIf																					;ckeck for timeout when no chracters recieved that gives an error
			IF (_Timer_Diff($hTimer)) > $iDelay Then												;check on time limit
				ExitLoop																			;with no known Feedback terminator
			EndIf																					;just time out
		EndIf
		IF $NextChar <> 0 Then																		;build recieved characters for debugging
			$ASCII = $ASCII & Chr($NextChar)														;but don't add null as it is a string terminator
		EndIf																						;and prevents @CRLF added from being printed
	WEnd
	_DebugMess("4",'Feedback Text Cleared: ' & $ASCII)
	IF StringLen($Mess) > 0 Then
		$Mess = '4X4 Matrix Switch Communications Error' & @CRLF & $Mess							;Add first common line to error
		MsgBox($MB_ICONERROR,$g_Title,$Mess)														;Display the Message
		SetError(1)
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sends Opens Serial Port to Switch retrieves current settings
; Parameters:		$Out	- String in space seperated hex values ie: '05 55 19 09 77'
; Returns:			Noting but sets GUI status line if any error sending command is incountered
; 					Will Abort program id Hex string contains invalid values outside 0-255 range
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _4X4Send($Out)
	_DebugMess("4","Command sent to 4X4: " & $Out)													;Trace out the Command about to be sent
	Local $aBytes = StringSplit($Out," ")															;provide string as shown in documentation ie. 88 30 01 FF
	Local $nBytes = $aBytes[0]																		;covert individual values into string array
	IF $nBytes = 0 Then Exit																		;No values then just exit with no error
	$Struct1 = DllStructCreate("byte[" & $nBytes & "]")												;Create an array to pass to the DLL allows zero bytes to be sent
	For $n = 1 To $nBytes 																			;Put the bytes into the DLL data struct
		$nByte = Dec($aBytes[$n], $NUMBER_AUTO)														;Convert individual HEX values in interger values
		IF $nByte < 0 OR $nByte > 255 Then 															;verify all byte values are in range
			MsgBox($MB_ICONERROR,$g_Title,"Invalid Byte value in 4X4 command: " & $Out)				;errors here are only caused internally by code not formatting commands properly
			Exit																					;so we aboty program
		EndIf
		DllStructSetData($Struct1, 1, $nByte, $n)													;Place Byte into Array sent to DLL
	Next
	_CommSendByteArray(DllStructGetPtr($Struct1), $nBytes, 0)										;send the bytes
	IF @error Then 																					;error sending the bytes
		$gV_CamStatus = 'Error sending command to 4X4 Matrix Switch'
		$g_bStatus = True
	EndIf
EndFunc		;==>_4X4Send

; #FUNCTION# ====================================================================================================================
; Description:		Recieves FeedBack form 4X4 matrix switch
; Parameters:		$iLength	- Maxium length of command returned to expect if more characters recieved then error
; Returns:			ASCII text or Space seperated Hex values
;					Displays MsgBox if error encountered recieving text
; Globals			$g_Config	- Configuration file to load settings from
;					Retrieve time in Milliseconds to wait for reply default 0.5 seconds
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _4X4Get($iLength)
	Local $Mess 																					;Any Error Message
	Local $iDelay = IniRead($g_Config,'4X4','delay',1000)											;How long to wait for reply in Milliseconds
	Local $hTimer = TimerInit()																		;Start a timer
	Local $Trm = 'EE'																				;Terminator string in longer replys
	Local $nTrm = 1																					;Additional characters to recieve after Terminator string is seen
	Local $ASCII = ''																				;used to recieve ASCII text response

	While True																						;loop to read in ACK messages from camera
		$NextChar = _CommReadByte(0)																;Get a byte a string conversion of number (eg '0', '20') with no waiting
		IF @error Then																				;@error = 1 No character recieved
			IF @error = 2 Then
				$Mess = "DLL Call failed"
				ExitLoop
			EndIf
			IF (_Timer_Diff($hTimer)) > $iDelay Then												;check on time limit
				_DebugMess("4",'4X4 Feedback timeout')
				ExitLoop																			;with no known Feedback terminator
			EndIf																					;just time out
		Else
			$ASCII = $ASCII & Chr($NextChar)
			IF StringRight($ASCII,StringLen($Trm)) = $Trm Then 										;Feedback terminator character recieved exit
				_DebugMess("4",'4X4 Feedback Terminator seen')
				$iLength = StringLen($ASCII) + $nTrm
			EndIf
			IF StringLen($ASCII) = $iLength Then													;Expected number of characters recieved
				_DebugMess("4",'4X4 Feedback number of characters seen')
				ExitLoop
			EndIf
		EndIf
	WEnd
	IF StringLen($Mess) > 0 Then
		$Mess = '4X4 Matrix Switch Communications Error' & @CRLF & $Mess							;Add first common line to error
		MsgBox($MB_ICONERROR,$g_Title,$Mess)														;Display the Message
		SetError(1)
	EndIf
	_DebugMess("4","4X4 feedback ASCII: " & $ASCII)													;Debug the Inquiry Message
	Return $ASCII																					;Return the string in ASCII format
EndFunc		;==>_4X4Get

