#include-once

; #INDEX# =======================================================================================================================
; Title .........: _6X6
; COM Channel ...: 3
; AutoIt Version : 3.3.14
; Language ......: English
; Description ...: Control Of Krammer 6X6 Composite and Unbalanced Audio Matrix switch
;					for more details see files: AdditionalFiles\vs606xl.pdf
;												AdditionalFiles\6X6 Kramer Video Matrix Switch serial protocol.docx
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
#ce

; #CURRENT# =====================================================================================================================
; _6X6open			- Open serial Port to Switch and retrieves current settings
; _6X6clear			- Clears input buffer from 6X6 matrix Switch
; _6X6set			- Sets Selected Output to selected Input
; _6X6Send			- Send command to 6X6 switch via serial port opened by _6X6open
; _6X6Get			- gets a response to a command from the serial port

; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Opens Serial Port to 6X6 Kramer Switch retrieves current settings
; Parameters:		$aOutputs	- BYREF Array to place current settings into
; 					0 Based, Output 1 is element 0, Input 1 is expressed as 0
; Returns:			Nothing values passed back in BYREF array
; Globals			$g_Config	- Configuration file to load settings from
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6open(ByRef $aOutputs)
	_CommSwitch(3)																					;Use COM Channel 3 for 6X6 Matrix Switch
	_PortOpen("6X6")																				;Open Serial Port for Camera Control
	_6X6read($aOutputs)
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Reads current settings from switch
; Parameters:		$aOutputs	- BYREF Array to place current settings into
; 					0 Based Output A is element 0 and input 1 is expressed as 0
; Returns:			Nothing values passed back in BYREF array
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6read(ByRef $aOutputs)
	_CommSwitch(3)																					;Use COM Channel 3 for 6X6 Matrix Switch
	_6X6clear()																						;Clear any previous stuff in serial buffer
	$Cmd = '05 80 8X 81'
	$Mess = ''																						;Any error mess in here
	For $i = 0 to 5																					;retrieve input for each output
		_6X6send(StringReplace($Cmd,'X',Hex($i+1,1),1))												;Hex used to format to single character string
		Local $CmdBack  = _6X6get(4)																;Characters to recieve
		IF _6X6chck('45 80 8? 81 ',$CmdBack) Then													;_6X6get adds trailing space so add one here
			$Mess = 'Bad Format 6X6 Matrix Switch Feedback: ' & $CmdBack
			ExitLoop
		EndIf
		$iInput = StringMid($CmdBack,8,1)
		IF $iInput < 1 OR $iInput > 6 Then
			$Mess = 'Bad Input number 6X6 Matrix Switch: ' & $CmdBack
			ExitLoop
		EndIf
		$aOutputs[$i] = $iInput-1
	Next
	IF StringLen($Mess) Then
		$gV_CamStatus = 'Incorrect 6X6 Matrix Switch Feedback: ' & $CmdBack
		$g_bStatus = True
	EndIf
	_CommSwitch(1)																					;Switch back COM Channel to default camera
EndFunc		;==>_6X6open

; #FUNCTION# ====================================================================================================================
; Description:		Clears any pending data from 6X6 switch.
; Parameters:		None
; Returns:			Nothing will display Error dialogs if errors occur
; Author(s)			Paul verboom (pboom)
;
; Switch sends data in response to front panel key presses so unexpected dat can be there.
; Best clear any data before sending any commands we are looking for response to so we only get the data we expect.
; Will time out after 1 second if full buffer can not be claered
; Does not set COM channel assumded taht is don by calling function.
; ===============================================================================================================================
Func _6X6clear()
	Local $Mess 																					;Any Error Message
	While True																						;loop to read in ACK messages from camera
		$NextChar = _CommReadByte(0)																;Get a byte a string conversion of number (eg '0', '20') with no waiting
		IF @error Then																				;@error = 1 No character recieved
			IF @error = 2 Then
				$Mess = 'DLL Call failed'
				ExitLoop
			EndIf
			IF @error = 3 Then
				$Mess = 'Error ' & $NextChar
			EndIf
			ExitLoop
		EndIf
	WEnd
	IF StringLen($Mess) > 0 Then
		$Mess = '6X6 Matrix Switch Communications Error' & @CRLF & $Mess							;Add first common line to error
		MsgBox($MB_ICONERROR,$g_Title,$Mess)														;Display the Message
		SetError(1)
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Checks format of a command returned by 6X6 switch
; Parameters:		$Mask 	- Expected format of returned command characters in ? position are ignored
;					$Cmd	-
; Returns:			True (1) if $Cmd fails format check
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6chck($Mask,$Cmd)
	FOR $i = 1 to StringLen($Mask)
		IF StringMid($Mask,$i,1) = '?' Then
			$Cmd = StringReplace($Cmd,$i,'?',1)
		EndIf
	Next
	_DebugMess("6",'6X6 check Mask = "' & $Mask & '"')
	_DebugMess("6",'6X6 check Cmd = "' & $Cmd & '"')
	IF $Cmd <> $Mask Then
		Return 1
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sets selected Output to indicated Input
; Parameters:		$iOutputs	- Output number 0 based
;					$iInput		- Input number 0 based
; Returns:			Noting but sets GUI status line if any error sending command or unexpected response is returned
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6set($iOutput,$iInput)
	_CommSwitch(3)																					;Use Channel 3 for 6X6 switch
	_6X6clear()
	Local $Cmd = '01 8Y 8X 81'																		;Build command to send
	$Cmd = StringReplace($Cmd,'X',$iOutput+1,1)														;Array contents is zero based
	$Cmd = StringReplace($Cmd,'Y',$iInput+1,1)														;correct to 1 based used by switch
	_6X6Send($Cmd)																					;Covert to byte values and send
	Local $CmdBck  = _6X6get(4)																		;Maxium characters to recieve
	IF _6X6chck('41 8? 8? 81 ',$CmdBck) then														;_6X6get adds trailing space so add one here
		$Mess = 'Invalid 6X6 Matrix Switch Command Back: ' & $CmdBck								;Read the command back verify switch setting was made
	Else
		IF StringMid($CmdBck,5,1) <> $iInput + 1 OR StringMid($CmdBck,8,1) <> $iOutput + 1 Then
			$Mess = '6X6 Matrix Switch Command Back incorrect switch set: ' & $CmdBck
		EndIf
	EndIf
	IF StringLen($Mess) Then
		$gV_CamStatus = $Mess
		$g_bStatus = True
	EndIf
	_CommSwitch(1)																					;Always switch back to COM for cameras
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Sends Opens Serial Port to Switch retrieves current settings
; Parameters:		$Out	- String in space seperated hex values ie: '05 55 19 09 77'
; Returns:			Noting but sets GUI status line if any error sending command is incountered
; 					Will Abort program id Hex string contains invalid values outside 0-255 range
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6Send($Out)
	_DebugMess("6","Command sent to 6X6: " & $Out)													;Trace out the Command about to be sent
	Local $aBytes = StringSplit($Out," ")															;provide string as shown in documentation ie. 88 30 01 FF
	Local $nBytes = $aBytes[0]																		;covert individual values into string array
	IF $nBytes = 0 Then Exit																		;No values then just exit with no error
	$Struct1 = DllStructCreate("byte[" & $nBytes & "]")												;Create an array to pass to the DLL allows zero bytes to be sent
	For $n = 1 To $nBytes 																			;Put the bytes into the DLL data struct
		$nByte = Dec($aBytes[$n], $NUMBER_AUTO)														;Convert individual HEX values in interger values
		IF $nByte < 0 OR $nByte > 255 Then 															;verify all byte values are in range
			MsgBox($MB_ICONERROR,$g_Title,"Invalid Byte value in 6X6 command: " & $Out)				;errors here are only caused internally by code not formatting commands properly
			Exit																					;so we aboty program
		EndIf
		DllStructSetData($Struct1, 1, $nByte, $n)													;Place Byte into Array sent to DLL
	Next
	_CommSendByteArray(DllStructGetPtr($Struct1), $nBytes, 0)										;send the bytes
	IF @error Then 																					;error sending the bytes
		$gV_CamStatus = 'Error sending command to 6X6 Matrix Switch'
		$g_bStatus = True
	EndIf
EndFunc

; #FUNCTION# ====================================================================================================================
; Description:		Recieves FeedBack form 6X6 matrix switch
; Parameters:		$iLength	- Length of bytes to recieve only this number recieved
; Returns:			Space seperated Hex values
;					Displays MsgBox if error encountered recieving text
; Globals			$g_Config	- Configuration file to load settings from
;					Retrieve time in Milliseconds to wait for reply default 0.5 seconds
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _6X6get($iLength)
	Local $Mess 																					;Any Error Message
	Local $iDelay = IniRead($g_Config,'6X6','delay',500)											;How long to wait for reply in Milliseconds
	Local $hTimer = TimerInit()																		;Start a timer
	Local $iLenCnt = 0
	Local $Hex = ''																					;used to build the response

	While True																						;loop to read in ACK messages from camera
		$NextChar = _CommReadByte(0)																;Get a byte a string conversion of number (eg '0', '20') with no waiting
		IF @error Then																				;@error = 1 No character recieved
			IF @error = 2 Then
				$Mess = "DLL Call failed"
				ExitLoop
			EndIf
			IF (_Timer_Diff($hTimer)) > $iDelay Then												;check on time limit must be done when no characters recieved
				;$Mess = 'Time Out recieving characters'
				ExitLoop																			;with no known Feedback terminator
			EndIf																					;just time out
		Else
			$Hex = $Hex & Hex($NextChar,2) & " "
			$iLenCnt += $iLenCnt
			IF $iLenCnt = $iLength Then	ExitLoop															;Have we recieved expected characters
		EndIf
	WEnd
	IF StringLen($Mess) > 0 Then
		$Mess = '6X6 Matrix Switch Communications Error' & @CRLF & $Mess							;Add first common line to error
		MsgBox($MB_ICONERROR,$g_Title,$Mess)														;Display the Message
		SetError(1)
	EndIf
	_DebugMess("6","6X6 Command back: " & $Hex)															;Debug the Inquiry Message
	Return $Hex																						;Return the string in HEX format
EndFunc

