#include-once
#include <GamePadEx.au3>							;Library for GamePad access

; #INDEX# =======================================================================================================================
; Title .........: _GAME
; AutoIt Version : 3.3.14.5
; Language ......: English
; Description ...: Implements Pan Tilt and Zoom Pad areas in GUI
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
Implements Gamepad/ Joystick control of Cameras
Allows the cameras to be Pan and Tilted Zoom with variable speed using Joystick/GamePad
Uses GamePadEx UDF by Shafayat see;
https://www.autoitscript.com/forum/topic/116958-gamepadex-udf/
#ce

; #CURRENT# =====================================================================================================================
; _GameSetup		- Intilize variables used to implement GamePad Control
; _GameProcess		- Processing to do in Main Loop
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Intilize variables used to implement GamePad control in Map Variable $gG_
; Parameters:		None
; Returns:			Nothing
; Globals			$g_Config	- Configuration file to load settings from
;					$gG_		- any variable with this prefix
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _GameSetup()

	;---------------------------
	;-- Calculate Knee Values --
	;---------------------------
	#cs
	Below Knee value 0-100 the ratio established by Speed and Knee value is applied to the KneeSpeed
	$iXspd
	---------  X 	$gG_iKneeSpd
	$gG_iKnee
	25
	-- 	* 	10 	= 5
	50
	Above Knee value amount above knee value is used with the above knee range to establish a ratio
	The ratio is applied to the full above knee speed range and then added to the top below knee speed
	($iXspd	- $gG_iKnee)
	--------------- 	*  	(100 - $gG_iKneeSpd)	+ 	$gG_iKneeSpd
	(100 - $gG_iKnee)
	(75 - 50)
	---------			* 	(100 - 25)	+ 25 	= 62
	(100 -50)

	(100 - 50)
	----------			* 	(100 -25)	+ 25 	= 100
	(100 - 50)
	#ce
	Global $gG_iKnee = IniRead($g_Config,"GamePad","Knee",50)							;Knee Point 0-100 under which differnet low rate is applied
	Global $gG_iKneeSpd = IniRead($g_Config,"GamePad","KneeSpeed",10)					;Top speed of low rate 0-100
	Global $gG_DeadZone = IniRead($g_Config,"GamePad","DeadZone",100)					;Dead Zone movement within this zone does enter HGamePad Mode

	;--------------------
	;-- Game Pad Stuff --
	;--------------------
	Global $gG_hGamePad = GPExInitialize() 												;Get a handle
	Global $gG_iGameID = IniRead($g_Config,"GamePad","ID",0) 							;Allow configuration to alter GamePad Monitored default = 0
	Global $gG_bGamePresent = False														;Flag by default no GamePad
	$aGameData = GPExGetRawData($gG_hGamePad, $gG_iGameID)								;Get all GamePad Data
	FOR $n = 0 to ubound($aGameData) -1 												;If any value is greater that zero than GamePad is present
		IF $aGameData[$n] > 0 Then $gG_bGamePresent = True
	Next
	IF $gG_bGamePresent Then
		_DebugMess("j","GamePad present")
	Else
		_DebugMess("j","GamePad NOT present")
	EndIf
	Global $gG_bZm = False							;Flag Zoom is in effect, if so turns it off first when dropping out of GamePad Mode
	Global $gG_bPOVrelease = False					;POV must be seen in OFF position before acting on it
	Global $gG_bJoyMode = False 					;Flag currently processing GamePad data
	Global $gG_iXspdLst = 0 						;Holds last GamePad value seen determines if there is movement
	Global $gG_iYspdLst = 0							;Holds signed interger that indicates direction as well as speed
	Global $gG_iZspdLst = 0							;signed interger converted to absolute value for all speed calculation
EndFunc		;==>_GameSetup

; #FUNCTION# ====================================================================================================================
; Description:		Processes Joystick movement activity
; Parameters:		None
; Returns:			Nothing
; Globals			$mG			- Map variable values are assigned to and retained during calls
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _GameProcess()
	IF $gG_bGamePresent Then
		$testrawdata = GPExGetRawData($gG_hGamePad, $gG_iGameID)
		#cs
		Get almost every data possible from the gamepad. Returns in an array with 8 elements.
		0 - X Axis - PAN  (0-65535) MidPoint 32767
		1 - Y Axis - Tilt (0-65535)MidPoint 32767
		2 - Z Axis - ZOOM absolute (0-65535) MidPoint 32767
		3 - R Axis - Zoom Adjust (0-65535) MidPoint 32767
		4 - U Axis
		5 - V Axis
		6 - Point of View - TopHat
			off = 65535
			up = 0
			down = 18000
			left = 27000
			right = 9000
		7 - Buttons - 0 = no button pressed
		#ce

		;------------------------
		;-- GamePad processing --
		;------------------------
		IF ABS(INT($testrawdata[0] - 32767)) < $gG_DeadZone Then					;Check for Dead Zone
			$iXspd = 0																;Within Dead Zone no X movement
			$XDir = 'Stop'															;Dead Zone subtracted from maxium posible values and value used
		Else																		;+Up -Down	value 0 to 100 ABS must be taken before sending to VISA
			$iXspd = INT((ABS($testrawdata[0] - 32767) - $gG_DeadZone) / (32767 -$gG_DeadZone) * 100)
			IF $testrawdata[0] > 32767 Then
				$XDir = 'Right'
			Else
				$XDir = 'Left'
			EndIf
		EndIf

		IF ABS(INT($testrawdata[1] - 32767)) < $gG_DeadZone Then					;Check for Dead Zone
			$iYspd = 0																;Within dead Zone no Y Movement
			$YDir = 'Stop'															;Dead Zone subtracted from maxium posible values and value used
		Else																		;+Up -Down	value 0 to 100 ABS must be taken before sending to VISA
			$iYspd = INT((ABS($testrawdata[1] - 32767) - $gG_DeadZone) / (32767 -$gG_DeadZone) * 100)
			IF $testrawdata[1] > 32767 Then
				$YDir = 'Up'
			Else
				$YDir = 'Down'
			EndIf
		EndIf

		IF ABS(INT($testrawdata[2] - 32767)) < $gG_DeadZone Then					;Check for Dead Zone
			$iZspd = 0																;Within Dead Zoom NO zoom
			$ZmDir = 'Stop'															;Dead Zone subtracted from maxium posible values and value used
		Else																		;-Wide +Tele Speed 0-100 ABS must be taken before sending to VISA
			$iZspd = INT((ABS($testrawdata[3] - 32767) - $gG_DeadZone) / (32767 -$gG_DeadZone) * 100)
			IF $testrawdata[3] > 32767 Then
				$ZmDir = 'Tele'
			Else
				$ZmDir = 'Wide'
			EndIf
		EndIf

		;-----------------------------------------
		;-- adjust lower rates with Knee values --
		;-----------------------------------------
		_DebugMess("k", ' PreKnee X=' & $iXspd & ' ' & $XDir  & " Y="  & $iYspd & ' ' & $YDir & ' Z=' & $iZspd & ' ' & $ZmDir)
		IF $iXspd <> 0 Then															;Only do calculation if not in dead zone
			IF $iXspd < $gG_iKnee Then
				$iXspd = $iXspd / $gG_iKnee  * $gG_iKneeSpd
			Else
				$iXspd = ($iXspd -$gG_iKnee)/ (100 - $gG_iKnee)
				$iXspd = $iXspd * (100 - $gG_iKneeSpd)
				$iXspd = $iXspd + $gG_iKneeSpd
			EndIf
			$iXspd = int($iXspd)
		EndIf
		IF $iYspd <> 0 then
			IF $iYspd < $gG_iKnee Then
				$iYspd = $iYspd / $gG_iKnee  * $gG_iKneeSpd
			Else
				$iYspd = ($iYspd - $gG_iKnee)/ (100-$gG_iKnee) * (100 - $gG_iKneeSpd) + $gG_iKneeSpd
			EndIf
			$iYspd = Int($iYSpd)
		EndIf
		IF $iZspd <> 0 Then
			IF $iZspd < $gG_iKnee Then
				$iZspd = $iZspd / $gG_iKnee  * $gG_iKneeSpd
			Else
				$iZspd = ($iZspd - $gG_iKnee)/ (100-$gG_iKnee) * (100 - $gG_iKneeSpd) + $gG_iKneeSpd
			EndIf
			$iZspd = INT($iZSpd)
		EndIf
		_DebugMess("k", 'PostKnee X=' & $iXSpd & ' ' & $XDir  & " Y="  & $iYspd & ' ' & $YDir & ' Z=' & $iZspd & ' ' & $ZmDir)

		;------------------------
		;-- Exit GamePad Mode --
		;------------------------
		IF $gG_bJoyMode Then																		;Are we in GamePad contyrol mode
			IF $iXspd = 0 AND $iYspd = 0 AND $iZspd = 0 Then										;no movement drop out of this mode
				IF $gG_bZm Then 																	;If a ZOOM is in effect Queue Zoom Stop command first
					_DebugMess("j","GamePadreleased ZOOM in effect stop that first.")
					_VscZoomStop()
					_VscPanTiltStop()
				Else
					_DebugMess("j","GamePadreleased ZOOM NOT in effect stop Pan/Tilt first.")
					_VscPanTiltStop()
					_VscZoomStop()
				EndIf
				$gG_bJoyMode = False
				$gV_CamStatus = ''																	;Clear the on screen labels regarding activity
				$gV_PanLbl = ''
				$gV_TltLbl = ''
				$gV_ZoomLbl = ''
				$g_bStatus = True																	;Force Status Update
				$gG_bZm = False																		;Zoom No longer in effect
				$gG_iXspdLst = 0																	;Clear values so we don't drop back into GamePad mode
				$gG_iYspdLst = 0
				$gG_iZspdLst = 0

			EndIf
		EndIf

		;-------------------------------------------
		;-- Movement in GamePad starts that mode --
		;-------------------------------------------
		IF $iXspd <> $gG_iXspdLst OR $iYspd <> $gG_iYspdLst AND (NOT $gV_bWait) Then				;have Pan/Tilt values changed then Queue up command
																									;But only if NOT waiting on another command to complete ACK
			_VscPanTlt($XDir,$iXSpd,$YDir,$iYSpd)													;Send Pan Tilt command via VISCA module
			_DebugMess("j", 'X=' & $iXspd & ' ' & $XDir  & " Y="  & $iYspd & ' ' & $YDir)
			IF NOT $gG_bJoyMode Then																;Set GamePad Mode if not already set
				$gG_bJoyMode = True
				$gV_CamStatus = 'GamePad'															;indicate JoyStick mode
				_DebugMess("j","GamePad mode activated.")
			EndIf
			$g_bStatus = True
			$gG_iXspdLst = $iXspd																	;Values seen this time comparied to next time through
			$gG_iYspdLst = $iYspd																	;used to detect changes in GamePad control position
		EndIf

		IF $gG_iZspdLst <> $iZspd  AND (NOT $gV_bWait) Then										;Has zoom values changed if so Queue up command
																									;But only if NOT waiting on another command to complete ACK
			_DebugMess("z", 'Zoom ' & $ZmDir & ' ' & Abs($iZSpd))									;trace out values
			_VscZoom($ZmDir,$iZSpd)															;Use VISCA module to send out the actual command
			_DebugMess("k", ' Z=' & $iZspd & ' ' & $ZmDir)
			IF NOT $gG_bJoyMode Then																;Set GamePad Mode if not already set
				$gG_bJoyMode = True
				$gV_CamStatus = 'GamePad'															;indicate JoyStick mode
				_DebugMess("j","GamePad mode activated.")
			EndIf
			$gG_bZm = True
			$g_bStatus = True
			$gG_iZspdLst = $iZspd																	;record this as last Zoom speed so chnage can be detected
		EndIf
	EndIf
EndFunc		;==>_GameProcess

; #FUNCTION# ====================================================================================================================
; Description:		Processes GamePad/Joystick Buttons only done if no waiting on commands
; Parameters:		None
; Returns:			Nothing sets Macro Commands to perform functions
; Globals			$mG			- Map variable values are assigned to and retained during calls
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _GameButtons()
	IF $gG_bGamePresent Then																		;Gamepad can be used to select things
		$testrawdata = GPExGetRawData($gG_hGamePad, $gG_iGameID)									;6 - Point of View - TopHat
																									;	off = 65535
																									;	up = 0
																									;	down = 18000
																									;	left = 27000
																									;	right = 9000
																									;7 - Buttons
																									; 	0 = no button pressed
																									;	1 = Trigger
		IF $testrawdata[6] = 27000 AND $gG_bPOVrelease Then											;POV left move selected camera left or decrease
			$n = $iCam -1																		;move selected camera left
			IF $n = 0 Then $n = $iCamFnd															;if at zero wrap around
			_McrCmdAdd('CameraSelect|' & $n)														;Have macro processor actually change the camera
			$gG_bPOVrelease = False																	;Prevent rapitly selecting next POV must be released first
		ElseIf $testrawdata[6] = 9000 AND $gG_bPOVrelease Then										;POV right move selected camera right or increase
			$n = $iCam + 1
			IF $n > $iCamFnd Then $n = 1
			_McrCmdAdd('CameraSelect|' & $n)														;Have macro processor actually change the camera
			$gG_bPOVrelease = False																	;Prevent rapitly selecting next POV must be released first
																									;Preset highlighter not a MACRO action but Trigger that
																									;Selects Preset is one that Uses actual name
		ElseIf $testrawdata[6] = 0 AND $gG_bPOVrelease Then											;POV UP
			$i = _GUICtrlListBox_GetCurSel($hPresets)												;Get selected Preset if any selected
			IF $i > 0 and UBound($aPrst) > 0 Then													;Not on first preset and we have presets								s
				$iPrstUpdate = $i - 1																;Move up one
				$gG_bPOVrelease = False
			ElseIf $i = -1 And Ubound($aPrst) > 0 Then												;No preset selected and we have presets
				$iPrstUpdate = Ubound($aPrst)-1														;Select bottom preset
				$gG_bPOVrelease = False
			EndIf
		ElseIf $testrawdata[6] = 18000 AND $gG_bPOVrelease Then										;POV DOWN
			$i = _GUICtrlListBox_GetCurSel($hPresets)												;Get selected Preset if any selected
			IF $i > -1 AND $i < Ubound($aPrst)-1 Then												;Preset is selected and is less than bottom one
				$iPrstUpdate = $i + 1																;then increase by one
				$gG_bPOVrelease = False
			ElseIf $i = -1 AND Ubound($aPrst) > 0 Then												;Preset NOT selected and we have presets
				$iPrstUpdate = 0																	;Select Top Preset
				$gG_bPOVrelease = False
			EndIf
		ElseIf $testrawdata[6] = 65535 Then
			$gG_bPOVrelease = True																	;You must release POV to allow another selection with it
		EndIf
		IF $testrawdata[7] <> 0 Then																;Other GamePad buttons pressed
			_DebugMess('b','GamePad Button: ' & $testrawdata[7])									;debug out the number
			IF $testrawdata[7] = 1 Then																;Trigger Pressed Recall Selected Preset
				Local $PrstName = GUICtrlRead($hPresets)
				IF StringLen($PrstName) > 0 Then
					_McrCmdAdd('PresetRecall|' & $PrstName)
				EndIf
			EndIf
		EndIf
	EndIf
EndFunc		;==>_GameButtons

; #FUNCTION# ====================================================================================================================
; Description:		Processes GamePad/Joystick Buttons only done if no waiting on commands
; Parameters:		None
; Returns:			Nothing sets Macro Commands to perform functions??
; Globals			$mG			- Map variable values are assigned to and retained during calls
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _GameEnd()
	;Function doesn't work with MAP variable but it just sets the Handle to Zero so do that here
	GPExTerminate($gG_hGamePad) 																		;Close gamepad handle for good manners.
	;$gG_hGamePad = 0
EndFunc		;==>_GameEnd