#include-once
#include <GDIPlus.au3>					;Drawing lines in Switch Matrix images
#include <ScreenCapture.au3>			;capture drawn switch matrix
#include <Array.au3>					;for Array Display used is Debugging

; #INDEX# =======================================================================================================================
; Title .........: Matrix
; AutoIt Version : 3.3.14.5
; Language ......: English
; Description ...: Functions to provide conytrol for a Matrix style video/audio switches.
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
Provides controls for Matrix style video/audio switches.
Control provides a visual matrix representation
Matrix switches have multiple input and output channels
Inputs accept audio/video signals in
Outputs provide selected audio/video as an output.
A single input can be feed to multiple outputs.
But multiple inputs cannot be feed to a single output.
Each output is connected to a single selected input.
Each output is represented by a element in a single deminsion array.
The value in the element determines what input has been selected.

INI file contains defination of Matrix number of inputs outputs which axis is output
and spacing around matrix left for labelling.
#ce

; #CURRENT# =====================================================================================================================
; _Matrix
; _MatrixClick
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
;Description:		Draws a Matrix Diagram given defination in INI file
;Parameters:		$idControl	   	- Id of the picture control being used for the Matrix Diagram, Handle of Window derived from this control ID
;					$ControlName	- used to find correct section in INI file also used as name of Bimap file to load or create
;					$aSelected		- Array with input for each output, each output is element in array, resized by this function
;					$aVpos			- returns a array of pixel positions lines across used to check cowardinates of mouse clicks
;					$aHpos			- returns an array of pixel positions lines down used to check cowardinates of mouse clicks
;Returns:			0	- Everything worked
;					1	- Created default bitmap file, Not really an error displays message to user to that effect
;Globals			$g_Config		- Configuration file to load settings from
;Author(s)			Paul verboom (pboom)
;
;Can cause program abort if window not located
;Looks for BMP bitmap file with name in $ControlName to load into GUI Picture control.
;If BMP file is not found one is created with the required intersecting lines.
;BMP file can then be edited adding required labelling and it will used in all future runs.
; ===============================================================================================================================
Func _Matrix($idControl, $ControlName, BYREF $aSelected, BYREF $aVpos, BYREF $aHpos)

	Local $hWindow = _Winapi_GetParent(GUICtrlGetHandle($idControl))								;figure out window handle from Control ID

	Local $ImageFileName = @ScriptDir & '\' & $ControlName & '.bmp'									;Determine name of image file
	Local $bNew = NOT(FileExists($ImageFileName))													;set flag if it does not exist so we build a sample one
	If NOT $bNew Then																				;image file exist load into the GUI picture control
		GUICtrlSetImage($idControl,$ImageFileName)
	EndIf

	$aCrd = WinGetPos($hWindow)																		;$aCrd[0] = X positon window
	If @error Then																					;$aCrd[1] = Y position window
		MsgBox($MB_ICONERROR,$g_Title,"Unable to get Window position for: " & $ControlName)		;errors here are only caused internally by code not formatting commands properly
		Exit																						;so we abort program
	EndIf																							;$aCrd[3] = Height
	$aSize = WinGetClientSize($hWindow)																;$aSize[0] = Width client area (area not including boarders)
	If @error Then																					;$aSize[1] = Height Client area
		MsgBox($MB_ICONERROR,$g_Title,"Unable to get Client Window size for: " & $ControlName)		;errors here are only caused internally by code not formatting commands properly
		Exit																						;so we abort program
	EndIf

	Local $aCntrlAbs[4]																				;absolute position of Control Matrix used to save bitmap of Matrix drawn
	$iBrd = ($aCrd[2] -$aSize[0]) / 2																;Determine size of boarder left right and botttom
																									;Get the position for the Control Matrix on the form
	$aCntrl = ControlGetPos($hWindow, "", $idControl)												;$aCntrl[0] = X position of switch matrix within client area of form form
																									;$aCntrl[1] = Y position of switch matrix within client area of form form
																									;$aCntrl[2] = Width switch matrix
																									;$aCntrl[3] = Height switch matrix
																									;Calculate the Absolute position required to get bitmap Matrix drawn
	$aCntrlAbs[0] = $aCrd[0] + $aCntrl[0] + $iBrd													;Left Edge
	$aCntrlAbs[2] = $aCntrlAbs[0] + $aCntrl[2] 														;Right Edge
	$aCntrlAbs[1] = $aCrd[1] + $aCntrl[1]  + $aCrd[3] - $aSize[1] - $iBrd							;Top Edge
	$aCntrlAbs[3] = $aCntrlAbs[1] + $aCntrl[3] 														;Bottom Edge

	_GDIPlus_Startup()																				;Create resources for doing graphics
	Local $hGraphic = _GDIPlus_GraphicsCreateFromHWND($hWindow)
	Local $hPen = _GDIPlus_PenCreate(0xFF000000, 2)
	Local $hBrush =  _GDIPlus_BrushCreateSolid(0xFF0D2EF7) 											;(0xFFE0A0FF) ;color format AARRGGBB (hex)

	_GDIPlus_GraphicsDrawRect ( $hGraphic, $aCntrl[0], $aCntrl[1], $aCntrl[2], $aCntrl[3])			;Draw rectangle around CONTROL graphic to verify we have correct cowardinates

	Local $iHchan = IniRead($g_Config,$ControlName,'col',-1)										;Number of Channels on side of matrix (rows)
	Local $iVchan = IniRead($g_Config,$ControlName,'row',-1)										;Number of channels at top of the matrix (columns)
	Local $iHoffset = IniRead($g_Config,$ControlName,'left',0)										;Space along the left side & top for labeling
	Local $iVoffset = IniRead($g_Config,$ControlName,'top',0)										;Labelling is added to the BitMap used for control
	Local $iEdge = IniRead($g_Config,$ControlName,'edge',0)											;space between right/bottom and boarder
	Local $iDotFactor = IniRead($g_Config,$ControlName,'DotFactor',1.25)							;factor used to reduce size of DOT below spacing of rows columns
	$RowSwtch = IniRead($g_Config,$ControlName,'Output','col')										;True = Use rows for output otherwise use Columns
	Local $bOutRow = StringLeft($RowSwtch,3) = 'row'												;remove any comments from comparison

	IF $iHchan = -1 OR $iVchan = -1 Then Return 1													;verify we have row & col values otherwise things get wonky

	If $bOutRow Then
		ReDim $aSelected[$iVchan]
	Else
		ReDim $aSelected[$iHchan]
	EndIf
	ReDim $aVpos[$iVchan]																			;Position of channels in the Y Vertical direction in relation to client area
	ReDim $aHpos[$iHchan]																			;Position of channels in the X horizontal direction in relation to client area
	Local $iVspc = ($aCntrl[3] - $iVoffset - $iEdge) / ($iVchan)									;Determine spacing between rows
	Local $iHspc = ($aCntrl[2] - $iHoffset - $iEdge) / ($iHchan)									;space between columns
	Local $iDotRadius = INT($iVspc / $iDotFactor)													;determine size of dots

	For $i = 0 to $iHchan -1																		;Draw Vertical Lines for each channel at top
		$iX = $iHoffset + $iHspc * $i + ($iHspc/2)													;To draw vertical lines 1 & 3 parameter remain the same
		$aHpos[$i] = $aCntrl[0] + $iX																;Position of top channels in relation to control used to draw dots
		IF $bNew Then
			_GDIPlus_GraphicsDrawLine($hGraphic, $aCntrl[0] + $iX, $aCntrl[1] + $iVoffset, $aCntrl[0] + $iX, $aCntrl[1] + $aCntrl[3] - $iEdge, $hPen)
		EndIf
	Next

	For $i = 0 to $iVchan -1																		;Draw Horizontal lines for each channel at right
		$iY = $iVoffset + $iVspc * $i + ($iVspc/2)													;To draw horizontal lines 2 & 4 parameter remain the same
		$aVpos[$i] = $aCntrl[1] + $iY																;Position of side channels in relation to control used to draw dots
		IF $bNew Then
			_GDIPlus_GraphicsDrawLine($hGraphic, $aCntrl[0] + $iHoffset, $aCntrl[1] + $iY, $aCntrl[0] + $aCntrl[2] - $iEdge, $aCntrl[1] + $iY, $hPen)
		EndIf
	Next

	If $bNew Then																					;We are creaing image save it
		_GDIPlus_BrushDispose($hBrush)
		_GDIPlus_PenDispose($hPen)																	;We have drawn image required now shut down
		_GDIPlus_GraphicsDispose($hGraphic)															;Graphics resources used so we can get BitMap of it
		_GDIPlus_Shutdown()
		Sleep(2000)																					;Give Graphics resources chance to close so image becomes stable
		_ScreenCapture_Capture($ImageFileName,$aCntrlAbs[0],$aCntrlAbs[1],$aCntrlAbs[2],$aCntrlAbs[3])
		IniWrite($g_Config,$ControlName,'DotSize',$iDotRadius)
		Local $Mess = 'Image for matrix control has been created in file;' & @CRLF & $ImageFileName & @CRLF & 'Please edit it adding any required labelling'
		MsgBox($MB_OK,$g_Title,$Mess)
		_GDIPlus_Startup()																			;Now Rereate Graphics resources for doing graphics
		Local $hGraphic = _GDIPlus_GraphicsCreateFromHWND($hWindow)									;so we can draw the dots
		Local $hPen = _GDIPlus_PenCreate(0xFF000000, 2)
		Local $hBrush =  _GDIPlus_BrushCreateSolid(0xFF0D2EF7) 										;(0xFFE0A0FF) ;color format AARRGGBB (hex)
	EndIf

	IF $bOutRow Then
		For $iy = 0 to $iVchan -1																	;plot the connections (connect the dots)
			IF $aSelected[$iy] >=0 AND $aSelected[$iy]< $iHchan Then
				_GDIPlus_GraphicsFillEllipse($hGraphic, $aHpos[$aSelected[$iy]] - $iDotRadius/2 ,$aVpos[$iy]	- $iDotRadius/2, $iDotRadius , $iDotRadius,$hBrush)
			EndIf
		Next
	Else
		For $ix = 0 to $iHchan -1																		;plot the connections (connect the dots)
			IF $aSelected[$ix] >= 0 and $aSelected[$ix] < $iVchan Then
				_GDIPlus_GraphicsFillEllipse($hGraphic, $aHpos[$ix] - $iDotRadius/2 ,$aVpos[$aSelected[$ix]]	- $iDotRadius/2, $iDotRadius , $iDotRadius,$hBrush)
			EndIf
		Next
	EndIf
	_GDIPlus_BrushDispose($hBrush)																	;Final Release of Graphics resources used
	_GDIPlus_PenDispose($hPen)
	_GDIPlus_GraphicsDispose($hGraphic)
	_GDIPlus_Shutdown()

	If $bNew Then																					;We are creating image save it
		Return 1
	Else
		Return 0
	EndIf
EndFunc		;==>_Matrix

; #FUNCTION# ====================================================================================================================
;Description:		Updates vales in $aSelected based on user clicks on Matrix Diagram
;Parameters:		$idControl	   	- Id of the picture control being used for the Matrix Diagram, Handle of Window derived from this control ID
;					$ControlName	- used to find correct section in INI file
;					$aSelected		- Array with input for each output, each output is element in array, updated by this function
;					$aVpos			- array of pixel positions lines across used to check cowardinates of mouse clicks
;					$aHpos			- array of pixel positions lines down used to check cowardinates of mouse clicks
;					$bAll			- Return all changes even if no change detected from current array setting
;Returns:			0	- Nothing changed
;					1	- Something changed indicated in $iOutput and $iInput, $aSelected is also updated
;Globals			$g_Config		- Configuration file to load settings from
;Author(s)			Paul verboom (pboom)
;
;Looks at position of mouse and determines if user clicked on one of the intersecting lines
;If so sees if $aSelected array element needs to be updated, and does so and returns array element updated
; ===============================================================================================================================
Func _MatrixClick($idControl, $ControlName, BYREF $aSelected, $aVpos, $aHpos, BYREF $iOutput, BYREF $iInput, $bAll)
	Local $iDotRadius = IniRead($g_Config,$ControlName,'DotSize',10) /2 							;get dot Radius sized figured out when Bitmap is first created
	$RowSwtch = IniRead($g_Config,$ControlName,'Output','col')										;True = Use rows for output otherwise use Columns
	Local $bOutRow = StringLeft($RowSwtch,3) = 'row'												;remove any comments from comparison
	Local $hWindow = _Winapi_GetParent(GUICtrlGetHandle($idControl))								;figure out window handle from Control ID
	AutoItSetOption ('MouseCoordMode',2)															;get Mouse cowardinates relative client area of window
	Local $aMousePos = MouseGetPos()																;this is default mode throughout
	Local $iHfnd = -1
	Local $iVfnd = -1
	For $ix = 0 to ubound($aHpos) -1																;Was mouse on one of the vertical lines
		IF $aMousePos[0] >= ($aHpos[$ix] - $iDotRadius) AND $aMousePos[0] <= ($aHpos[$ix] + $iDotRadius) Then
			$iHfnd = $ix
			ExitLoop
		EndIf
	Next
	For $iy = 0 to ubound($aVpos) -1																;Was mouse on one of the horizontal lines
		IF $aMousePos[1] >= ($aVpos[$iy] - $iDotRadius) AND $aMousePos[1] <= ($aVpos[$iy] + $iDotRadius) Then
			$iVfnd = $iy
			ExitLoop
		EndIf
	Next
	IF $iHfnd > -1 AND $iVfnd > -1 Then																;Was on a intersection of lines
		IF $bOutRow Then
			IF $aSelected[$iVfnd] <> $iHfnd OR $bAll Then
				$aSelected[$iVfnd] = $iHfnd
				$iOutput = $iVfnd
				$iInput = $iHfnd
				_Matrix($idControl,$ControlName,$aSelected,$aVpos,$aHpos)
				Return 1
			EndIf
		Else
			IF $aSelected[$iHfnd] <> $iVfnd OR $bAll Then
				$aSelected[$iHfnd] = $iVfnd
				$iOutput = $iHfnd
				$iInput = $iVfnd
				_Matrix($idControl,$ControlName,$aSelected,$aVpos,$aHpos)
				Return 1
			EndIf
		EndIf
		_Matrix($idControl,$ControlName,$aSelected,$aVpos,$aHpos)
	EndIf
	Return 0
EndFunc


