#include-once
#include <Timers.au3>								;Time outs when waiting for camera response
#include <AutoItConstants.au3>						;Needed for Hex conversion function
#include <WinAPIGdi.au3>							;_WinAPI_PtInRectEx - Determines whether the specified point lies within the specified rectangle
#include <GDIPlus.au3>								;Drawing rectangles arrow Arrow HotSpots for debugging

; #INDEX# =======================================================================================================================
; Title .........: _PAD
; AutoIt Version : 3.3.14.5
; Language ......: English
; Description ...: Implements Pan Tilt and Zoom Pad areas in GUI
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
Implements a virtual type of Joystick using Mouse drag gestuers in a defined area of the GUI.
Allows the cameras to be Pan and Tilted with variable speed using mouse gestures
#ce

; #CURRENT# =====================================================================================================================
; _PadSetup		- Intilize variables used to implement Pan/Tilt and Zoom Pads
; _PadProcess	- Processing to do in Main Loop
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Intilize variables used to implement Pan/Tilt and Zoom Pads in Map Variable $gP_
; Parameters:		None
; Returns:			Nothing
; Globals			$g_Config	- Configuration file to load settings from
;					$gP_		- any variable with this prefix
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _PadSetup()
	Global $gP_iZmAxis = IniRead($g_Config,'Pad','ZoomAxis',1)	;Axis ZOOM change takes place on Horizontal x = 0, Vertical y = 0
	Global $gP_aPosLstX = -1						;Last Position of Mouse used to determine changes
	Global $gP_aPosLstY = -1						;cannt make an array part of MAP variable
	Global $gP_iZmLst								;last position of mouse on ZOOM pad only concerned with change in one axis
	Global $gP_bPanMode = False						;In Pan/Tilt control via mouse mode
	Global $gP_bArwMode = False						;In Pan adjustment via Arrow HotSpots
	Global $gP_bZmMode = False						;In ZOOM control via mouse mode
	Global $qp_bArwZmMode = False					;In Zoom Adjustment via arrow spots
	Global $gP_iOrginalMouse = 0					;orginal Mouse Cursor
	Global $gP_hUserDLL = DllOpen("user32.dll")		;Open handle to DLL used to check if mouse button is down
	Global $gP_aArw[4][6]							;Cowardinates of arrow hotspots
													;1st element hotspot
													;	0 = left
													;	1 = up
													;	2 = right
													;	3 = down
													;2nd element
													;	0 = x-coordinate of the upper-left corner of the rectangle.
													;	1 = y-coordinate of the upper-left corner of the rectangle.
													;	2 = x-coordinate of the lower-right corner of the rectangle.
													;	3 = y-coordinate of the lower-right corner of the rectangle.
													;	4 = Pan or Tilt speed value 0 to 100
													;	5 = Pan or Tilt direction L R U or D, for left Right Up or Down
	Global $gP_aArwZm[2][6]							;Cowardinates of arrow hotspots for Zoom
													;1st element hotspot
													;	0 = left
													;	1 = up
													;	2 = right
													;	3 = down
													;2nd element
													;	0 = x-coordinate of the upper-left corner of the rectangle.
													;	1 = y-coordinate of the upper-left corner of the rectangle.
													;	2 = x-coordinate of the lower-right corner of the rectangle.
													;	3 = y-coordinate of the lower-right corner of the rectangle.
													;	4 = Zoom speed value 0 to 100
													;	5 = Zoom direction T or W, for Tele or Wide

	Local $iArwSz = 0 + IniRead($g_Config,'Pad','ArrowSize',40)										;Size of arrow and hotspot
	Local $iArwBrd = 0 + IniRead($g_Config,'Pad','ArrowBoarder',5)									;Space between edge of adjustment pad and arrow
	Local $iArwSpd  = 0 +IniRead($g_Config,'Pad','ArrowSpeed',25)									;Speed to use when making PAN/TILT adjustments with arrows
	Local $iArwZmSpd = 0 +IniRead($g_Config,'Pad','ArrowZoomSpeed',25)								;Spped to use when making ZOOM adjustments with arrows

	$gP_aArw[0][4] = $iArwSpd							;set Speed to use for each direction
	$gP_aArw[1][4] = $iArwSpd
	$gP_aArw[2][4] = $iArwSpd
	$gP_aArw[3][4] = $iArwSpd
	$gP_aArw[0][5] = 'L'								;set the direction each arrow causes
	$gP_aArw[1][5] = 'U'
	$gP_aArw[2][5] = 'R'
	$gP_aArw[3][5] = 'D'

	$gP_aArwZm[0][4] = $iArwZmSpd
	$gP_aArwZm[1][4] = $iArwZmSpd
	$gP_aArwZm[0][5] = 'Tele'
	$gP_aArwZm[1][5] = 'Wide'

	Local $aPad = ControlGetPos($hForm, "", $hPTpad)	;Position and size of Pan/Tilt Pad
	Local $iX, $iY

	$iX = $aPad[0] + $iArwBrd							;Calculate Left X corner of Left Arrow HotSpot
	$iY = $aPad[1] + $aPad[3]/2 - $iArwSz/2				;Calculate Top Y corner of Left Arrow HotSpot
	$gP_aArw[0][0] = $iX								;Set All Left Arrow Boundaries
	$gP_aArw[0][2] = $iX + $iArwSz
	$gP_aArw[0][1] = $iY
	$gP_aArw[0][3] = $iY + $iArwSz
	$gP_aArw[2][1] = $iY								;Same Y vertical boundries for Right Arrow
	$gP_aArw[2][3] = $iY + $iArwSz

	$iX = $aPad[0]  + $aPad[2] - $iArwSz - $iArwBrd		;Calculate Left X corner of Right Arrow Boundry
	$gP_aArw[2][0] = $iX								;Set left & right X boundries for Right Arrow
	$gP_aArw[2][2] = $iX + $iArwSz

	$iX = $aPad[0] + $aPad[2]/2 - $iArwSz/2				;Calculate Left X corner of Up Arrow HotSpot
	$iY = $aPad[1] + $iArwBrd							;Calculate Top Y Corner of Up arrow HotSpot
	$gP_aArw[1][0] = $iX								;Set All Up Arrow Boundaries
	$gP_aArw[1][2] = $iX + $iArwSz
	$gP_aArw[1][1] = $iY
	$gP_aArw[1][3] = $iY + $iArwSz
	$gP_aArw[3][0] = $iX								;Same Horizontal X boundries for Down Arrow
	$gP_aArw[3][2] = $iX + $iArwSz

	$iY = $apad[1] + $aPad[3] - $iArwSz - $iArwBrd		;Calculate Top Y Corner for Down Arrow HotSpot
	$gP_aArw[3][1] = $iY								;Set Top Bottom Y Boundries for Down Arrow
	$gP_aArw[3][3] = $iY + $iArwSz

	Local $aPad = ControlGetPos($hForm, "", $hZOOMpad)	;Position and size of Zoom Pad
	Local $iX, $iY

	$iX = $aPad[0] + $aPad[2]/2 - $iArwSz/2				;Calculate Left X corner of Tele Zoom HotSpot
	$iY = $aPad[1] + $iArwBrd							;Calculate Top Y corner of Tele Zoom HotSpot
	$gP_aArwZm[0][0] = $iX								;Set All Left Arrow Boundaries
	$gP_aArwZm[0][2] = $iX + $iArwSz
	$gP_aArwZm[0][1] = $iY
	$gP_aArwZm[0][3] = $iY + $iArwSz
	$gP_aArwZm[1][0] = $iX								;Same Y vertical boundries for Right Arrow
	$gP_aArwZm[1][2] = $iX + $iArwSz

	$iY = $aPad[1] + $aPad[3] - $iArwSz - $iArwBrd		;Calculate Top Y Corner for Wide Zoom HotSpot
	$gP_aArwZm[1][1] = $iY
	$gP_aArwZm[1][3] = $iY + $iArwSz

	IF StringInStr($g_Debug,"h") Then																;Draw Boxes around arrows in Pad DOESN'T WORK YET
		Local $hWindow = _Winapi_GetParent(GUICtrlGetHandle($hPTpad))								;figure out window handle from Control ID
		$aCntrl = ControlGetPos($hWindow, "", $hPTpad)												;$aCntrl[0] = X position of switch matrix within client area of form form
																									;$aCntrl[1] = Y position of switch matrix within client area of form form
																									;$aCntrl[2] = Width switch matrix
																									;$aCntrl[3] = Height switch matrix
		_GDIPlus_Startup()																			;Create resources for doing graphics
		Local $hGraphic = _GDIPlus_GraphicsCreateFromHWND($hPTpad)
		Local $hPen = _GDIPlus_PenCreate(0xFF000000, 2)
		Local $hBrush =  _GDIPlus_BrushCreateSolid(0xFF0D2EF7) 										;(0xFFE0A0FF) ;color format AARRGGBB (hex)

		For $i = 0 to 3																				;draw box around each hotspot
			_GDIPlus_GraphicsDrawRect ( $hGraphic, $gP_aArw[$i][0], $gP_aArw[$i][1], $iArwSz, $iArwSz)
		Next

		_GDIPlus_BrushDispose($hBrush)																;Final Release of Graphics resources used
		_GDIPlus_PenDispose($hPen)
		_GDIPlus_GraphicsDispose($hGraphic)
		_GDIPlus_Shutdown()
	EndIf

	AutoItSetOption ('MouseCoordMode',2)			;Throughout get Mouse cowardinates relative client area of window

EndFunc		;==>_PadSetup

; #FUNCTION# ====================================================================================================================
; Description:		Processes any activity of in Pan/Tilt and Zoom Pads
; Parameters:		None
; Returns:			Nothing
; Globals			$mC			- Map variable values are assigned to and retained during calls
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _PadProcess()
	;-------------------------
	;-- Enter Pan/Tilt Mode --
	;-------------------------
																									;NOT in Pan or Arrow Mode and Left Mouse Button Down and Program is active
	If (NOT $gP_bPanMode) AND (NOT $gP_bArwMode) AND _IsPressed("01", $gP_hUserDLL) AND WinGetHandle('[active]') =  $hForm Then
		Local $aPos = MouseGetPos()																	;Get mouse position relative to Client Area

		FOR $i = 0 to 3																				;Check for clicks on Arrows Hotspots
			IF _WinAPI_PtInRectEx($aPos[0],$aPos[1],$gP_aArw[$i][0],$gP_aArw[$i][1],$gP_aArw[$i][2],$gP_aArw[$i][3]) Then
				$gP_iOrginalMouse = MouseGetCursor()												;Save orginal Mouse Cursor and change to cross arrows
				GUISetCursor(9, $GUI_CURSOR_OVERRIDE , $hForm)
				Local $aTrap[4]																		;Trap Mouse to the Arrow Button
				$aTrap[0] = $gP_aArw[$i][0]
				$aTrap[1] = $gP_aArw[$i][1]
				$aTrap[2] = $gP_aArw[$i][2] - $gP_aArw[$i][0]
				$aTrap[3] = $gP_aArw[$i][3] - $gP_aArw[$i][1]
				_TrapMouse($atrap)
				$gP_bArwMode = True																	;Set mode to Arrow click adjust
				$iPanTltSpd = $gP_aArw[$i][4]														;set the direction and speed from those stored in array
				$PanTltDir = $gP_aArw[$i][5]
				_DebugMess("p", "Arrow Pan Tilt " & $PanTltDir & ' ' & $iPanTltSpd)					;Everything else is set so enter PanTilt Mode
				_VscPanTlt($PanTltDir,$iPanTltSpd,$PanTltDir,$iPanTltSpd)							;Send Pan Tilt command via VISCA module also sets Status in GUI
				ExitLoop
			EndIf
		Next


		IF NOT $gP_bArwMode Then																	;Didn't drop into Arrow adjust mode check for Pad control
			Local $aPad = ControlGetPos($hForm, "", $hPTpad)										;Position and size of Pan/Tilt Pad

			;IF $aPos[0] > $aPad[0] AND $aPos[0] < $aPad[0] + $aPad[2] AND $aPos[1] > $aPad[1] AND $aPos[1] < $aPad[1] + $aPad[3] Then
			IF _WinAPI_PtInRectEx($aPos[0],$aPos[1],$aPad[0],$aPad[1],$aPad[0] + $aPad[2],$aPad[1] + $aPad[3]) Then
				_DebugMess("m","Left Mouse Buton was pressed in Pan/Tilt Pad.")
				$gP_iOrginalMouse = MouseGetCursor()												;Save orginal Mouse Cursor and change to cross arrows
				GUISetCursor(9, $GUI_CURSOR_OVERRIDE , $hForm)
				_TrapMouse($aPad)																	;Trap Mouse to the ZoomPad and recenter it
				$gP_aPosLstX = $aPos[0]																;Record starting position of mouse
				$gP_aPosLstY = $aPos[1]
				$gV_CamStatus = 'Pan/Tilt Pad'														;Pan/Tilt Pad mode
				$g_bStatus = True																	;Force update of Status in GUI
				$gP_bPanMode = True
			EndIf
		EndIf
	EndIf

	;-------------------------------
	;-- Check for Pan/Tile Change --
	;-------------------------------
	If $gP_bPanMode AND _IsPressed("01", $gP_hUserDLL) AND (NOT $gV_bWait) Then						;In Pan Mode mouse button still pressed and not waiting for ACK
		Local $aPos = MouseGetPos()																	;We are free to send additional commands
		IF $gP_aPosLstX <> $aPos[0] OR $gP_aPosLstY <> $aPos[1] Then								;Has Mouse been moved lets see if we need to send new command

			;------------------------------
			;-- Prepare Pan/Tilt Command --
			;------------------------------
			Local $aPad = ControlGetPos($hForm, "", $hPTpad)										;Position and size of Pan/Tilt Pad
			$nPanHlf = ($aPad[2]-2)/2																;Full range either direction minus a bit so we can get fully to 100
			$iPanSpd = ABS(INT(($aPad[0] - $aPos[0] + $nPanHlf) / $nPanHlf * 100))  				;value 0 to 100, direction read from $PanDir
			IF $aPos[0] > $aPad[0] + $nPanHlf Then
				$PanDir = 'Right'
			Else
				$PanDir = 'Left'
			EndIf
			$nTltHlf = ($aPad[3] -2) /2																;Full range either direction minus a bit so we can get fully to 100
			$iTltSpd = ABS(INT(($aPad[1] - $aPos[1] + $nTltHlf) / $nTltHlf * 100))					;value 0 to 100 direction read from $TltDir
			IF $aPos[1] > $aPad[1] + $nTltHlf Then
				$TltDir = 'Down'
			Else
				$TltDir = 'Up'
			EndIf																					;trace out values
			_DebugMess("p", "Pad Drag Pan " & $PanDir & ' ' & $iPanSpd & " Tilt "  & $TltDir & ' ' & $iTltSpd)
			_VscPanTlt($PanDir,$iPanSpd,$TltDir,$iTltSpd)											;Send Pan Tilt command via VISCA module
			$gP_aPosLstX = $aPos[0]																	;New Last position of Mouse
			$gP_aPosLstY = $aPos[1]																	;so we can tell if it moved.
		EndIf
	EndIf

	;-------------------
	;-- Exit Pan Mode --
	;-------------------
	IF 	($gP_bPanMode OR $gP_bArwMode) AND (NOT _IsPressed("01", $gP_hUserDLL)) then				;Mouse button released drop out of pan mode or arrow adjust mode
		_VscPanTiltStop()																			;Stop Pan Tilt Operation
		GUISetCursor($gP_iOrginalMouse, $GUI_CURSOR_OVERRIDE , $hForm)								;Restore mouse cursor, remove mouse trap
		_MouseTrap()
		$gP_bPanMode = False																		;Drop out of Pan Mode allow it to be enterred again
		$gP_bArwMode = False																		;Drop out or arrow adjust mode allow it to be enterred again
		$gV_CamStatus = ''
		$gV_PanLbl = ''
		$gV_TltLbl = ''
		$g_bStatus = True
		_DebugMess("m","Left Mouse Buton was released from PanTilt Pad.")
	EndIf

	;---------------------
	;-- Enter ZOOM Mode --
	;---------------------
    If (NOT $gP_bZmMode)  AND (NOT $qp_bArwZmMode) AND _IsPressed("01", $gP_hUserDLL) AND WinGetHandle('[active]') =  $hForm Then	;NOT in Zoom Mode and Left Mouse Button Down
		Local $aPos = MouseGetPos()																	;Get mouse position relative to Client Area

		FOR $i = 0 to 1																				;Check for clicks on Arrows Hotspots
			IF _WinAPI_PtInRectEx($aPos[0],$aPos[1],$gP_aArwZm[$i][0],$gP_aArwZm[$i][1],$gP_aArwZm[$i][2],$gP_aArwZm[$i][3]) Then
				$gP_iOrginalMouse = MouseGetCursor()												;Save orginal Mouse Cursor and change to cross arrows
				GUISetCursor(9, $GUI_CURSOR_OVERRIDE , $hForm)
				Local $aTrap[4]																		;Trap Mouse to the Arrow Button
				$aTrap[0] = $gP_aArwZm[$i][0]
				$aTrap[1] = $gP_aArwZm[$i][1]
				$aTrap[2] = $gP_aArwZm[$i][2] - $gP_aArwZm[$i][0]
				$aTrap[3] = $gP_aArwZm[$i][3] - $gP_aArwZm[$i][1]
				_TrapMouse($atrap)
				$qp_bArwZmMode = True																;Set mode to Arrow click adjust
				$iZmSpd = $gP_aArwZm[$i][4]															;set the direction and speed from those stored in array
				$ZmDir = $gP_aArwZm[$i][5]
				_DebugMess("p", "Arrow Zoom " & $ZmDir & ' ' & $iZmSpd)								;Everything else is set so enter PanTilt Mode
				_VscZoom($ZmDir,$iZmSpd)															;Use VISCA module to send out the actual command
				ExitLoop
			EndIf
		Next


		IF NOT $qp_bArwZmMode Then																	;Didn't drop into Arrow adjust mode check for Pad control
			Local $aZm = ControlGetPos($hForm, "", $hZOOMpad)										;Position and size of the Zoom Pad
			IF $aPos[0] > $aZm[0] AND $aPos[0] < $aZm[0] + $aZm[2] AND $aPos[1] > $aZm[1] AND $aPos[1] < $aZm[1] + $aZm[3] Then
				_DebugMess("z","Left Mouse Buton was pressed in Zoom Pad.")
				$gP_iOrginalMouse = MouseGetCursor()												;Save orginal Mouse Cursor and change to cross arrows
				GUISetCursor(9, $GUI_CURSOR_OVERRIDE , $hForm)
				_TrapMouse($aZm)																	;Trap Mouse to the ZoomPad and recenter it
				$gP_iZmLst = $aZm[$gP_iZmAxis]														;Record starting position of mouse only concerned with one axis
				$gV_CamStatus = 'Zoom Pad'															;indicate ZOOM pad mode
				$gP_bZmMode = True
			EndIf
		EndIf
	EndIf

	;---------------------------
	;-- Check for ZOOM Change --
	;---------------------------
	If $gP_bZmMode AND _IsPressed("01", $gP_hUserDLL) AND (NOT $gV_bWait) Then					;In Pan Mode mouse button still pressed and not waiting for ACK
		Local $aZm = ControlGetPos($hForm, "", $hZOOMpad)											;Position and size of the Zoom Pad
		Local $aPos = MouseGetPos()																	;We are free to send additional commands
		IF $gP_iZmLst <> $aPos[$gP_iZmAxis] Then													;Has Mouse been moved lets see if we need to send new command
			$nZmHlf = ($aZm[2 + $gP_iZmAxis] -2 )/2													;Calculate full range in either direction -2 so we can get full offset
			$iZmSpd = ABS(INT(($aZm[$gP_iZmAxis] - $aPos[$gP_iZmAxis] + $nZmHlf) / $nZmHlf * 100))	;-Wide +Tele Speed 0-100
			IF $aPos[$gP_iZmAxis] > $aZm[$gP_iZmAxis] + $nZmHlf Then
				$ZmDir = 'Wide'
			Else
				$ZmDir = 'Tele'
			EndIf
			_DebugMess("z", 'Zoom ' & $ZmDir & ' ' & $iZmSpd)										;trace out values
			_VscZoom($ZmDir,$iZmSpd)																;Use VISCA module to send out the actual command
			$gP_iZmLst = $aPos[$gP_iZmAxis] 														;New Last position of Mouse in selected axis so we can tell if it moved.
		EndIf
	EndIf

	;--------------------
	;-- Exit ZOOM Mode --
	;--------------------
	IF ($gP_bZmMode OR $qp_bArwZmMode) AND (NOT _IsPressed("01", $gP_hUserDLL)) then				;Mouse button released drop out of ZOOM mode
		_VscZoomStop()																				;Stop ZOOM mode
		GUISetCursor($gP_iOrginalMouse, $GUI_CURSOR_OVERRIDE , $hForm)								;Restore mouse cursor, remove mouse trap
		_MouseTrap()
		$qP_bArwZmMode = False																		;Drop out ofd Zoom Mode both Arrow Hotspots
		$gP_bZmMode = False																			;and mouse drag
		$gV_CamStatus = ''																			;Clear on screen indicators
		$gV_ZoomLbl = ''
		$g_bStatus = True
		_DebugMess("z","Left Mouse Button was released from ZOOM Pad.")
	EndIf
EndFunc		;==>_PadProcess

; #FUNCTION# ====================================================================================================================
;Description:		Confines mouse movement to Control user has clicked down on
;Parameters:		$aControl	   	- contains cowardinates of control 0=Left 1=Top 2=Width 3=Height
;Returns:			Nothing
;Globals			$hForm			- Handle of Window used for GUI
;Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _TrapMouse($aControl)
	Local $aWinSize = WinGetPos($hForm)														;Window Position and size
																							;0 = Left X
																							;1 = Top Y
																							;2 = Width
																							;3 = Height
	Local $aClntSize = WinGetClientSize($hForm)												;Get size of client area of Form does not include boarders
																							;0 = Width
																							;1 = Height
	Local $iWinBrd = ($aWinSize[2] -$aClntSize[0]) / 2										;Determine size of boarder left right and botttom
	Local $aTrap[4]																			;Calculate absolute cowardinates of Pan/Tilt Pad for Mouse Trap
	$aTrap[0] = $aWinSize[0] + $aControl[0] + $iWinBrd										;Left Edge
	$aTrap[2] = $aTrap[0] + $aControl[2] 													;Right Edge
	$aTrap[1] = $aWinSize[1] + $aControl[1]  + $aWinSize[3] - $aClntSize[1] - $iWinBrd		;Top Edge
	$aTrap[3] = $aTrap[1] + $aControl[3] 													;Bottom Edge
	_MouseTrap($aTrap[0], $aTrap[1], $aTrap[2], $aTrap[3])									;Trap mouse to Control
	MouseMove($aControl[0] + ($aControl[2]/2),$aControl[1] + ($aControl[3]/2),0)			;Recenter Mouse on Pad so we start at zero movement
EndFunc		;==>_TrapMouse

