#include-once

; #INDEX# =======================================================================================================================
; Title .........: _ProjB
; COM Channel ...: 4
; AutoIt Version : 3.3.14
; Language ......: English
; Description ...: Control EPSON PowerLite 4650  Projector
; Author(s):       Paul Verboom (pboom)
; ===============================================================================================================================

#cs
#ce

; #CURRENT# =====================================================================================================================
; _ProjBopen		- Open serial Port to Projector
; _ProjBcmd			- Sends command
; ===============================================================================================================================

; #FUNCTION# ====================================================================================================================
; Description:		Opens Serial Port to Projector
; Parameters:		None
; Returns:			Nothing
; Globals			$g_Config	- Configuration file to load COm port settings from
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _ProjBopen()
	_CommSwitch(5)																					;Use COM Channel 2 for 4X4 Matrix Switch
	_PortOpen("ProjB")																				;Open Serial Port for Camera Control
	_CommSwitch(1)																					;Revert port Cameras
EndFunc		;==>_ProjBopen

; #FUNCTION# ====================================================================================================================
; Description:		Sends command to projector
; Parameters:		$Command to send one of following
;						Standby
;						ON
;						VGA
;						Video
;						HDMI
;						MUTE ON
;						MUTE OFF
;Returns:			Nothing
; Author(s)			Paul verboom (pboom)
; ===============================================================================================================================
Func _ProjBcmd($Cmd)
	Local $SndCmd																					;Translate Macro Command plain
	Switch $Cmd																						;text to Projector Command
		Case 'Standby'
			$SndCmd = 'pwr off'
		Case 'ON'
			$SndCmd = 'pwr on'
		Case 'VGA'
			$SndCmd = 'source 11'
		Case 'Video'
			$SndCmd = 'source 41'
		Case 'HDMI'
			$SndCmd = 'source 30'
		Case 'MUTE ON'
			$SndCmd = 'mute on'
		Case 'MUTE OFF'
			$SndCmd = 'mute off'
	EndSwitch
	IF StringLen($SndCmd) > 0 Then																	;If any command then send it
		_CommSwitch(5)																				;Use COM Channel 5 for Projector B
		_DebugMess("q",'Command sent Projector B:' & $SndCmd)										;debug out command sent before Crriage return added
		$SndCmd = $SndCmd & @CR																		;and terminating Carriage Return
		_CommSendString($SndCmd,0)																	;Send the command
		_CommSwitch(1)																				;Revert COM to Cameras
		$gV_CamStatus = 'Project B ' & $Cmd
		$g_bStatus = True
	EndIf
EndFunc		;==>_ProjBcmd

