#include-once

; #INDEX# =======================================================================================================================
; Title .........: _web
; AutoIt Version : 3.3.14
; Language ......: English
; Description ...: Simple web server allows remote playback of macros
; Author(s)......: Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================

#cs
Orginal Source from Autoit Forum, using basic version
	https://www.autoitscript.com/forum/topic/68851-powerful-http-server-in-pure-autoit/

Resources:
    Internet Assigned Number Authority - all Content-Types: http://www.iana.org/assignments/media-types/
    World Wide Web Consortium - An overview of the HTTP protocol: http://www.w3.org/Protocols/

Credits:
    Manadar for starting on the webserver.
    Alek for adding POST and some fixes
    Creator for providing the "application/octet-stream" MIME type.

Web Server supports multiple sessions and GET command only no Post.
Will deliver standard file types so can deliver a standard web page
Get request with no file extension are treated as Macro Names and returned to calling program
Connection to Server with no subpage selected produces index page that list all macros with clickable links.
Macro can be selected by giving Web Server address in following format
	http::\\Computername\macroname
#ce

; #CURRENT# =====================================================================================================================
; _WEBintial		- Intilize webserver opens TCP port
; _WEBprocess		- Processing for web server to do in Main Loop
; ===============================================================================================================================


; #FUNCTION# ====================================================================================================================
; Description:		Initializes the webserver
; Parameters:		None
; Returns:			Noting but sets GUI status line if any error sending command is incountered
; Globals			$g_Config	- Configuration file to load settings from
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _WEBintial()
																									;Global to be available for other functions in this module
	Global $gW_RootDir = @ScriptDir & "\www" 														;The absolute path to the root directory of the server.
	Global $gW_iMainSocket																			;Socket set up for web server
	Global $gW_ServerName																			;Name of serve rplaced in HTTP Headers

																									;used to initialize stuff
	Local $sIP = @IPAddress1 																		;Local IP address as determined by AutoIt
	Local $iPort = IniRead($g_Config,'Web','Port',80)												; the listening port
	Local $iMaxUsers = IniRead($g_Config,'Web','MaxUsers',15)										; Maximum number of users/sessions that can simultaneously get

	Global $gW_aSocket[$iMaxUsers] 																	;Creates an array to store all the possible users/sessions
	Global $gW_aBuffer[$iMaxUsers] 																;All these users/sessions have buffers when sending/receiving
																									;Create buffer for each user/session
	For $x = 0 to UBound($gW_aSocket)-1 															;Fills the entire socket array with -1 integers, so that the server knows they are empty.
		$gW_aSocket[$x] = -1
	Next

	TCPStartup() 																					;AutoIt needs to initialize the TCP functions
	$gW_iMainSocket = TCPListen($sIP,$iPort) 														;create main listening socket
	If @error Then 																					;if you fail creating a socket
		$gV_CamStatus = 'Unable to create a socket for webserver on port ' & $iPort					;send a error message to the status line
		$g_bStatus = True
		Return
	EndIf

	Local $sServerAddress = "http://" & $sIP & ":" & $iPort & "/"									;Trace out the Server created message
	_DebugMess("w", "Server created on " & $sServerAddress)

EndFunc		;==>_WEBintial

; #FUNCTION# ====================================================================================================================
; Description:		Processing for web server to do in Main Loop
; Parameters:		None
; Returns:			Macro to Play, Name not verified
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _WEBprocess()
	$iNewSocket = TCPAccept($gW_iMainSocket) 														;Tries to accept incoming connections

    If $iNewSocket >= 0 Then 																		;Verifies that there actually is an incoming connection
        For $x = 0 to UBound($gW_aSocket)-1 														;Attempts to store the incoming connection
            If $gW_aSocket[$x] = -1 Then
                $gW_aSocket[$x] = $iNewSocket 														;store the new socket
                ExitLoop
            EndIf
        Next
    EndIf

    For $x = 0 to UBound($gW_aSocket)-1 															;A big loop to receive data from everyone connected
        If $gW_aSocket[$x] = -1 Then ContinueLoop 													; if the socket is empty, it will continue to the next iteration, doing nothing
        $sNewData = TCPRecv($gW_aSocket[$x],1024) 													;Receives a whole lot of data if possible
        If @error Then 																				;Client has disconnected
            $gW_aSocket[$x] = -1 																	;Socket is freed so that a new user may join
            ContinueLoop 																			;Go to the next iteration of the loop, not really needed but looks oh so good
        ElseIf $sNewData Then 																		;data received
            $gW_aBuffer[$x] &= $sNewData 															;store it in the buffer
            If StringInStr(StringStripCR($gW_aBuffer[$x]),@LF&@LF) Then 							;if the request has ended ..
                $sFirstLine = StringLeft($gW_aBuffer[$x],StringInStr($gW_aBuffer[$x],@LF)) 			;helps to get the type of the request
                $sRequestType = StringLeft($sFirstLine,StringInStr($sFirstLine," ")-1) 				;gets the type of the request
                If $sRequestType = "GET" Then 														;user wants to download a file or whatever ..
                    $sRequest = StringTrimRight(StringTrimLeft($sFirstLine,4),11) 					;let's see what file they actually wants
					If StringInStr(StringReplace($sRequest,"\","/"), "/.") Then 					;Disallow any attempts to go back a folder
						;_HTTP_SendError($gW_aSocket[$x]) 											; sends back an error
																									;Send back nothing just ignore the bad behavior
					Else
						If $sRequest = "/" Then 													;user has requested the root
							$sRequest = "/index.html" 												;instead of root we'll give him the index page
						EndIf
						$sRequest = StringReplace($sRequest,"/","\") 								;convert HTTP slashes to windows slashes, not really required because windows accepts both

						;----------------------------
						;-- Return Macro Requested --
						;----------------------------
						IF StringInStr($sRequest,'.') = 0 Then										;NO period so no file type must be a Macro selection
							$MacroName = StringReplace($sRequest,'\','')							;Strip any leading backslash
							;$MacroName = $sRequest
							Local $nMcrSel = _ArraySearch($aMcrStored,$MacroName,0,0,0,0,1,0)		;Look for Macro using name
							IF $nMcrSel = -1 Then $MacroName &= ' (Invalid Macro Name)'				;Macro Name not found add indication to Macro Name

							_DebugMess("w", "Macro selected: " & $MacroName)								;Trace out the macro selected
							_HTTP_SendIndex($gW_aSocket[$x],$gW_RootDir &  "/index.html",$MacroName)		;Send out the updated Index page
							$gW_aBuffer[$x] = "" 															;clears the buffer because we just used to buffer and
																											;did some actions based on them
							$gW_aSocket[$x] = -1 															;the socket is automatically closed so we reset the socket
																											;so that we may accept new clients
							Return $nMcrSel																	;Exit function and return verified number of macro requested

						;---------------------------
						;-- Process file requests --
						;---------------------------
						Else
							$sFileType = StringRight($sRequest,4) 									;determines the file type, so that we may choose what mine type to use

							If FileExists($gW_RootDir & "\" & $sRequest) Then 						;makes sure the file that the user wants exists
								Switch $sFileType

									Case "html", ".htm" 											;in case of normal HTML files
										IF StringInStr($sRequest, "index") > 0 Then					;If index is selected send dynamically built one
											_HTTP_SendIndex($gW_aSocket[$x],$gW_RootDir &  "/index.html","")
										ELSE
											_HTTP_SendFile($gW_aSocket[$x], $gW_RootDir & $sRequest, "text/html")
										EndIf
									Case ".css" 													;in case of style sheets
										_HTTP_SendFile($gW_aSocket[$x], $gW_RootDir & $sRequest, "text/css")
									Case ".jpg", "jpeg" 											;for common images
										_HTTP_SendFile($gW_aSocket[$x], $gW_RootDir & $sRequest, "image/jpeg")
									Case ".png" 													;another common image format
										_HTTP_SendFile($gW_aSocket[$x], $gW_RootDir & $sRequest, "image/png")
									Case Else 														;this is for .exe, .zip, or anything else that is not supported
																									;is downloaded to the client using a application/octet-stream
										_HTTP_SendFile($gW_aSocket[$x], $gW_RootDir & $sRequest, "application/octet-stream")
								EndSwitch
							Else
								_HTTP_SendFileNotFoundError($gW_aSocket[$x]) 						;File does not exist, so we'll send back an error..
							EndIf
						EndIf
					EndIf
                EndIf

                $gW_aBuffer[$x] = "" 																;clears the buffer because we just used to buffer and
																									;did some actions based on them
                $gW_aSocket[$x] = -1 																;the socket is automatically closed so we reset the socket
																									;so that we may accept new clients
            EndIf
        EndIf
    Next
	Return -1																						;No selected MACRO return not found value
EndFunc		;==>_WEBprocess

; #FUNCTION# ====================================================================================================================
; Description:		Converts any characters like %20 into space 8
; Parameters:		String to convert escaped URL characters
; Returns:			Parameter is by BYREf change made in orginal string
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _HTTP_ConvertString(ByRef $sInput)
    $sInput = StringReplace($sInput, '+', ' ')
    StringReplace($sInput, '%', '')
    For $t = 0 To @extended
        $Find_Char = StringLeft( StringTrimLeft($sInput, StringInStr($sInput, '%')) ,2)
        $sInput = StringReplace($sInput, '%' & $Find_Char, Chr(Dec($Find_Char)))
    Next
EndFunc		;==>_HTTP_ConvertString

; #FUNCTION# ====================================================================================================================
; Description:		Sends HTML data on X socket
; Parameters:		$hSocket	- socket number to send data on
;					$sHTML		- HTML data to send
;					$sReply		- Status used in packet header defaults '200 OK'
; Returns:			Nothing
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _HTTP_SendHTML($hSocket, $sHTML, $sReply = "200 OK")
    _HTTP_SendData($hSocket, Binary($sHTML), "text/html", $sReply)
EndFunc		;==>_HTTP_SendHTML

; #FUNCTION# ====================================================================================================================
; Description:		Prepares to send file on X socket
; Parameters:		$hSocket	- socket number to send data on
;					$sFileLoc	- Full name of file with path
;					$sMimeType	- File type placed in header
;					$sReply		- Status used in packet header defaults '200 OK'
; Returns:			Nothing
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _HTTP_SendFile($hSocket, $sFileLoc, $sMimeType, $sReply = "200 OK")
    Local $hFile, $sImgBuffer, $sPacket, $a

	_DebugMess("w", "Sending " & $sFileLoc & @CRLF)

    $hFile = FileOpen($sFileLoc,16)
    $bFileData = FileRead($hFile)
    FileClose($hFile)

    _HTTP_SendData($hSocket, $bFileData, $sMimeType, $sReply)
EndFunc		;==>_HTTP_SendFile

; #FUNCTION# ====================================================================================================================
; Description:		Sends the contents of file to X socket
; Parameters:		$hSocket	- socket number to send data on
;					$bData		- Contents of file to send
;					$sMimeType	- File type placed in header
;					$sReply		- Status used in packet header defaults '200 OK'
; Returns:			Nothing
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _HTTP_SendData($hSocket, $bData, $sMimeType, $sReply = "200 OK")
	$sPacket = Binary("HTTP/1.1 " & $sReply & @CRLF & _
    "Server: " & $g_Title & @CRLF & _
	"Connection: close" & @CRLF & _
	"Content-Lenght: " & BinaryLen($bData) & @CRLF & _
    "Content-Type: " & $sMimeType & @CRLF & _
    @CRLF)
    TCPSend($hSocket,$sPacket) ; Send start of packet

    While BinaryLen($bData) ; Send data in chunks (most code by Larry)
        $a = TCPSend($hSocket, $bData) ; TCPSend returns the number of bytes sent
        $bData = BinaryMid($bData, $a+1, BinaryLen($bData)-$a)
    WEnd

    $sPacket = Binary(@CRLF & @CRLF) ; Finish the packet
    TCPSend($hSocket,$sPacket)

	TCPCloseSocket($hSocket)
EndFunc		;==>_HTTP_SendData

; #FUNCTION# ====================================================================================================================
; Description:		Sends basic 404 the file not found error on X socket
; Parameters:		$hSocket	- socket number to 404 message
; Returns:			Nothing
; Author(s): 		Jos van Egmond (jvanegmond), edits by: Paul verboom (pboom)
; ===============================================================================================================================
Func _HTTP_SendFileNotFoundError($hSocket) ; Sends back a basic 404 error
	Local $s404Loc = $gW_RootDir & "\404.html"
	If (FileExists($s404Loc)) Then
		_HTTP_SendFile($hSocket, $s404Loc, "text/html")
	Else
		_HTTP_SendHTML($hSocket, "404 Error: " & @CRLF & @CRLF & "The file you requested could not be found.")
	EndIf
EndFunc		;==>_HTTP_SendFileNotFoundError

; #FUNCTION# ====================================================================================================================
; Description:		Sends Index File containing Activated Macro and Macro List
; Parameters:		$hSocket	- socket to send HTML to
;					$sFileLoc 	- HTML file to base sent file on
; Returns:			Nothing
;
; Fomate for Macro list
;	<a href="Pulpit">Pulpit</a><br>
;	<a href="Piano">Piano</a><br>
; ===============================================================================================================================
Func _HTTP_SendIndex($hSocket,$sFileLoc,$sRequest)
	$hFile = FileOpen($sFileLoc,0)
    $FileData = FileRead($hFile)
    FileClose($hFile)

	$FileData = StringReplace($FileData,'INSERT_ACTIVATEDMACRO',$sRequest)
	Local $MacroList
	For $i = 0 to UBound($aMcrStored)-1
		$Line = StringReplace('<a href="INSERT_NAME">INSERT_NAME</a><br>','INSERT_NAME',$aMcrStored[$i][0])
		$MacroList = $MacroList & $Line & @CRLF
	Next
	$FileData = StringReplace($FileData,'INSERT_AVAILABLEMACROS',$MacroList)

ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sFileLoc = ' & $sFileLoc & @CRLF) ;### Debug Console
ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $MacroList = ' & $MacroList & @CRLF) ;### Debug Console

	_HTTP_SendHTML($hSocket,$FileData)
EndFunc